/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Martin Rexa
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */
package cz.xelfi.quoridor.statistics;

import cz.xelfi.quoridor.webidor.Game;
import cz.xelfi.quoridor.webidor.GameId;
import cz.xelfi.quoridor.webidor.GameStatus;
import cz.xelfi.quoridor.webidor.User;
import cz.xelfi.quoridor.Move;
import java.util.HashMap;
import java.util.Map;
import java.util.List;
import java.util.ArrayList;
import javax.xml.bind.annotation.XmlAttribute;

/**
 *
 * @author Martin Rexa
 */
public class OpeningTreeNode {
    @XmlAttribute
    String hashCode;
    List<GameId> gameIds;
    List<GameId> finishedGames;
    Map<Move, OpeningTreeNode> children;

    public OpeningTreeNode(){
    }

    public OpeningTreeNode(String hashCode){
        this.hashCode = hashCode;
        gameIds = new ArrayList<GameId>();
        finishedGames = new ArrayList<GameId>();
        children = new HashMap<Move,OpeningTreeNode>();
    }

    public OpeningTreeNode addGame(Game game){
        gameIds.add(game.getId());
        return this;
    }

    public OpeningTreeNode addFinishedGame(Game game){
        finishedGames.add(game.getId());
        return this;
    }

    public OpeningTreeNode addGameId(GameId gameId){
        gameIds.add(gameId);
        return this;
    }

    public OpeningTreeNode addChild(Move move, OpeningTreeNode child){
        children.put(move, child);
        return this;
    }

    public OpeningNodeView getView(User user){
        return getView(user.getId());
    }

    public OpeningNodeView getView(String userId){
        OpeningNodeView view = new OpeningNodeView(hashCode);
        for(Map.Entry<Move, OpeningTreeNode> e: children.entrySet()){
            for(GameId gId: e.getValue().gameIds){
                if(User.canSee(gId, userId)){
                    view.processGame(e.getKey(), e.getValue().hashCode, gId);
                }
            }
        }
        for(GameId gId: finishedGames){
            if(User.canSee(gId, userId)){
                view.addFinishedGame(gId);
            }
        }
        return view;
    }

    public List<GameId> filterGames(String userId, GameStatus status){
        List<GameId> result = new ArrayList<GameId>();
        for(GameId gId: gameIds){
            if(User.canSee(gId, userId))
                if(gId.getStatus().equals(status))
                    result.add(gId);
        }
        return result;
    }

    public List<GameId> getGameIds(){
        return gameIds;
    }

//    @XmlAttribute
//    @XmlElement
    public String getHashCode(){
        return this.hashCode;
    }


    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        OpeningTreeNode other = (OpeningTreeNode)obj;
        return hashCode.equals(other.hashCode);
    }

    @Override
    public int hashCode() {
        return hashCode.hashCode();
    }

}
