/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */

package cz.xelfi.quoridor.statistics.resources;

import cz.xelfi.quoridor.statistics.EloList;
import cz.xelfi.quoridor.webidor.GameId;
import cz.xelfi.quoridor.webidor.GameStatus;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 *
 * @author Martin Rexa
 */
public final class Elo {
    private EloList list;
    private ArrayList<EloList> listHistory;
    private boolean firstGame;
    static long day = 1000 * 60 * 60 * 24;

    public Elo(){
        listHistory = new ArrayList<EloList>();
        list = new EloList();
        listHistory.add(list);
        firstGame = true;
    }

    public void processGame(GameId gId){
        GameStatus status = gId.getStatus();
        splitList(gId.getModified());
        if(status.equals(GameStatus.whiteWon)){
            list.putResult(gId.getWhite(), gId.getBlack());
        }else if(status.equals(GameStatus.blackWon)){
            list.putResult(gId.getBlack(), gId.getWhite());
        }
    }

    private void splitList(long gameModified){
        if(firstGame){
            firstGame = false;
            list.setEndDate(computeListEndDate(gameModified));
        }else{
            if(gameModified > list.getEndDate()){
                list = new EloList(list);
                listHistory.add(0,list);
                list.setEndDate(computeListEndDate(gameModified));
            }
        }
    }

    private long computeListEndDate(long modified){
        Calendar now = new GregorianCalendar();
        Calendar cModified = new GregorianCalendar();
        cModified.setTimeInMillis(modified);
        Calendar result = new GregorianCalendar();
        result.clear();
        result.set(Calendar.YEAR, cModified.get(Calendar.YEAR));
        result.set(Calendar.MONTH, cModified.get(Calendar.MONTH)+1);
        return result.getTimeInMillis()-1;
    }

    @GET
    @Path("list")
    @Produces({ MediaType.APPLICATION_JSON, MediaType.TEXT_XML })
    public EloList getList(){
        return list;
    }

    @GET
    @Path("list/{id}")
    @Produces({ MediaType.APPLICATION_JSON, MediaType.TEXT_XML })
    public EloList getHistoryList(@PathParam("id") Integer id){
        if(id == null)
            return null;
        if(id >= listHistory.size())
            return null;
        return listHistory.get(id);
    }

    @GET
    @Path("{username}")
    @Produces(MediaType.TEXT_PLAIN)
    public String getElo(@PathParam("username") String id){
        list.getElo(id);
        return Double.toString(list.getElo(id));
    }

}
