/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Martin Rexa
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */
package cz.xelfi.quoridor.statistics.resources;

import cz.xelfi.quoridor.IllegalPositionException;
import cz.xelfi.quoridor.webidor.Game;
import cz.xelfi.quoridor.webidor.GameId;
import cz.xelfi.quoridor.statistics.OpeningTree;
import cz.xelfi.quoridor.statistics.OpeningTreeNode;
import cz.xelfi.quoridor.statistics.OpeningNodeView;
import cz.xelfi.quoridor.webidor.GameStatus;
import java.util.List;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.core.MediaType;

/**
 *
 * @author Martin Rexa
 */
public class Openings {
    OpeningTree tree;

    public Openings(){
        tree = new OpeningTree();
    }

    public void processGame(Game game) {
        try{
            tree.processGame(game);
        }catch (IllegalPositionException e){
            e.printStackTrace();
        }
    }

    @GET
    @Path("{nodeId}")
    @Produces({ MediaType.APPLICATION_JSON, MediaType.TEXT_XML })
    public OpeningNodeView getNode(@QueryParam("loginID") @DefaultValue("") String loginId,
                         @PathParam("nodeId") String nodeId){
        OpeningTreeNode node = tree.getNode(nodeId);
        if(node == null)
            node = tree.getRoot();
        return node.getView(loginId);
    }

    @GET
    @Path("{nodeId}.check")
    @Produces({ MediaType.TEXT_PLAIN })
    public String checkNode(@QueryParam("loginID") @DefaultValue("") String loginId,
                         @PathParam("nodeId") String nodeId){
        OpeningTreeNode node = tree.getNode(nodeId);
        if(node == null)
            return "";
        if(node.getView(loginId).isEmpty())
            return "";
        else
            return nodeId;
    }

    @GET
    @Path("{nodeId}/{status}")
    @Produces({MediaType.APPLICATION_JSON, MediaType.TEXT_XML })
    public List<GameId> listGames(
        @DefaultValue("") @QueryParam("loginID") String loginId,
        @PathParam("nodeId") String nodeId,
        @DefaultValue("") @PathParam("status") String status
    ) {
        OpeningTreeNode node = tree.getNode(nodeId);
        if(node == null)
            return null;
        if("white".equals(status))
            return node.filterGames(loginId, GameStatus.whiteWon);
        else if("black".equals(status))
            return node.filterGames(loginId, GameStatus.blackWon);
        else
            return null;
    }
}
