/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Martin Rexa
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */
package cz.xelfi.quoridor.statistics;

import cz.xelfi.quoridor.Fence.Orientation;
import org.junit.Test;
import static org.junit.Assert.*;
import cz.xelfi.quoridor.Move;
import cz.xelfi.quoridor.webidor.Game;
import java.util.Map;

/**
 *
 * @author Martin Rexa
 */
public class OpeningsTest extends Object {
    
    public OpeningsTest() throws Exception {
    }

    @Test public void testMirrorMoves() throws Exception {
        assertEquals("bad symetric move", Move.NORTH.getMirrorMove(), Move.NORTH);
        assertEquals("bad symetric move", Move.SOUTH.getMirrorMove(), Move.SOUTH);
        assertEquals("bad symetric move", Move.WEST.getMirrorMove(), Move.EAST);
        assertEquals("bad symetric move", Move.EAST.getMirrorMove(), Move.WEST);
        assertEquals("bad symetric move", Move.fence('A', 1, Orientation.HORIZONTAL).getMirrorMove(), Move.fence('H', 1, Orientation.HORIZONTAL));
        assertEquals("bad symetric move", Move.fence('D', 3, Orientation.VERTICAL).getMirrorMove(), Move.fence('E', 3, Orientation.VERTICAL));
    }

    @Test public void testMirrorJump() throws Exception {
        Game g = new Game("w","b");
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.SOUTH, new java.util.Date());
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.SOUTH, new java.util.Date());
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.SOUTH, new java.util.Date());
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.EAST, new java.util.Date());
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.valueOf("WW"), new java.util.Date());
        g.apply("w", Move.RESIGN, new java.util.Date());
        OpeningTree t = new OpeningTree();
        t.processGame(g);
        OpeningTreeNode n = t.root;
        n = n.children.get(Move.NORTH);
        n = n.children.get(Move.SOUTH);
        n = n.children.get(Move.NORTH);
        n = n.children.get(Move.SOUTH);
        n = n.children.get(Move.NORTH);
        n = n.children.get(Move.SOUTH);
        n = n.children.get(Move.NORTH);
        n = n.children.get(Move.WEST);
        n = n.children.get(Move.NORTH);
        n = n.children.get(Move.valueOf("EE"));
        assertEquals("bad number of children", n.children.size(), 0);
        for(Map.Entry e: n.children.entrySet()){
            System.out.println(e.getKey());
        }
    }

    @Test public void testOpeningTreeNodes() throws Exception {
        OpeningTree t = new OpeningTree();
        assertEquals("bad number of nodes", t.nodes.size(), 1);
        Game g = new Game("w","b");
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.SOUTH, new java.util.Date());
        g.apply("w", Move.EAST, new java.util.Date());
        g.apply("b", Move.EAST, new java.util.Date());
        g.apply("w", Move.WEST, new java.util.Date());
        g.apply("b", Move.WEST, new java.util.Date());
        g.apply("w", Move.RESIGN, new java.util.Date());
        t.processGame(g);
        assertEquals("bad number of nodes", t.nodes.size(), 9);
        g = new Game("w","b");
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.SOUTH, new java.util.Date());
        g.apply("w", Move.fence('A', 1, Orientation.HORIZONTAL), new java.util.Date());
        g.apply("b", Move.fence('C', 1, Orientation.VERTICAL), new java.util.Date());
        g.apply("w", Move.RESIGN, new java.util.Date());
        t.processGame(g);
        assertEquals("bad number of nodes", t.nodes.size(), 13);
        g = new Game("w","b");
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.SOUTH, new java.util.Date());
        g.apply("w", Move.fence('C', 1, Orientation.VERTICAL), new java.util.Date());
        g.apply("b", Move.fence('A', 1, Orientation.HORIZONTAL), new java.util.Date());
        g.apply("w", Move.RESIGN, new java.util.Date());
        t.processGame(g);
        assertEquals("bad number of nodes", t.nodes.size(), 15);
        OpeningTreeNode n = t.root;
        n = n.children.get(Move.NORTH);
        n = n.children.get(Move.SOUTH);
        OpeningTreeNode n1 = n;
        n = n.children.get(Move.WEST);
        n = n.children.get(Move.WEST);
        n = n.children.get(Move.EAST);
        n = n.children.get(Move.EAST);
        assertEquals("different nodes", n.hashCode, n1.hashCode);
        n = n.children.get(Move.fence('F', 1, Orientation.VERTICAL));
        n = n.children.get(Move.fence('H', 1, Orientation.HORIZONTAL));
        assertEquals("bad number of children", n.children.size(), 0);
    }

}
